/* 
    grunt-dogescript
    https://github.com/Bartvds/grunt-dogescript

    Copyright (c) 2013 Bart van der Schoor
    Licensed under the MIT license.
*/

"use strict";

var rd = require('require-doge');
var doge = require('../lib/doge.djs');
var dogescript = require('dogescript');
var async = require('async');

function exported(grunt) {

    function error(messages) {
        if (messages instanceof Array) {
            messages = messages.join(' ');
        }
        grunt.log.error(messages);
    }

    function warn(messages) {
        if (messages instanceof Array) {
            messages = messages.join(' ');
        }
        grunt.log.warn(messages);
    }

    function line(messages) {
        if (messages instanceof Array) {
            var string = messages.join(' ');
            grunt.log.writeln(string);
        } else if (typeof messages === 'string') {
            grunt.log.writeln(messages);
        } else {
            grunt.log.writeln();
        }
    }

    function report(messages, success) {
        var message = doge.report(messages, success);
        line(message);
    }

    function getCompiler(options) {
        var dogescript = require('dogescript');
        if (options.compiler) {
            if (typeof options.compiler === 'function') {
                dogescript = options.compiler
            } else if (typeof options.compiler === 'string') {
                var src = options.compiler;
                try {
                    dogescript = require(src);
                } catch (e) {
                    line();
                    error('plz compilers', src);
                }
            }
        }
        return dogescript;
    }

    function compileDoge(compiler, file, options, callback) {
        var src = grunt.file.read(file.src);
        src = src.replace(/(?:\r\n)|\r/g, '\n');

        var res = compiler(src, options.beauty, options.trueDoge);
        if (res) {
            grunt.file.write(file.dest, res);
            callback();
        } else {
            var error = new Error('much error compile');
            callback(error);
        }
    }


    function task() {
        var options = {};
        options.beauty = true
        options.trueDoge = false
        options.compiler = null

        options = this.options(options);

        var done = this.async();
        var files = [];
        var fileCount = 0;

        var compiler = getCompiler(options);
        if (!compiler) {
            line();
            warn('missin doge!');
            line();
            done(false);
            return
        }

        // flatten list for sanity 
        this.files.forEach(function(f) {
            f.src.forEach(function(filePath) {
                var exists = grunt.file.exists(filePath);
                if (!exists) {
                    var res = doge.report(['exist', filePath.red, 'good', 'success', ], false);
                    line(res);
                    line();
                    warn('fantom doge!');
                    line();
                    return false
                }

                fileCount++
                var dest = f.dest;
                if (!dest) {
                    dest = filePath.replace(/.djs$/, '.js');
                }

                var file = {};
                file.src = filePath
                file.dest = dest
                file.options = options

                files.push(file);
            })
        })

        // sad no doge 
        if (fileCount === 0) {
            report(['input', 'source', 'files', 'empty', 'work', 'path', 'compile', 'empty', ], false);
            line();
            warn('silli doge!');
            line();
            done(false);
            return
        }

        // loop amaze 
        var success = 0;
        var failed = 0;

        function end(err) {
            if (err) {
                report(['task', 'dogescript', 'compile', 'error', ], false);
                line();
                warn('broke doge');
                line();
                done(false);
            } else if (success < fileCount || fileCount === 0) {
                report(['task', [success, ' success', ].yellow, 'compile', ('failed, ' + failed).red, ', error'], false);
                line();
                warn('bad doge!');
                line();
                done(false);
            } else {
                report(['task', [success, ' success', ].green, 'compile', ], true);
                line();
                grunt.log.ok();
                done(true);
            }
        }

        // TODO implement threaded doge 
        async.eachLimit(files, 1, function(file, callback) {
                compileDoge(compiler, file, options, function(err) {
                    if (err) {
                        warn(err);
                        report(['input', file.src.red, 'compile', 'error', ], false);
                        failed++
                    } else {
                        report(['input', file.src.green, 'compile', 'build', file.dest.cyan, 'code', ], true);
                        success++
                    }
                    line();
                    callback();
                })
            },
            end
        )
    }

    grunt.registerMultiTask('dogescript', 'doge compile codes', task);
}

module.exports = exported